/*
   A program which drops its privileges.  If it is setuid,
   it will drop its privileges to those of the user specified
   on the command line.
*/

#include <sys/types.h>
#include <unistd.h>
#include <stdio.h>
#include <pwd.h>
#include <errno.h>

int
main(int argc, char *argv[])
{
    struct passwd *new;
    int rc1, rc2;

    if (argc != 2) {
        fprintf(stderr, "Usage: %s user\n", argv[0]);
	exit(1);
    }

    printf("Before change:\n");
    printf("    Real UID: %5d; Effective UID: %5d\n", getuid(), geteuid());
    printf("    Real GID: %5d; Effective GID: %5d\n", getgid(), getegid());

    errno = 0;
    if ((new = getpwnam(argv[1])) == NULL) {
        fprintf(stderr, "User '%s' lookup failed: %s\n", argv[0],
	        strerror(errno));
	if (errno == 0) fprintf(stderr, "User not found\n");
	exit(1);
    }
    endpwent();

    if ((rc1 = setgid(new->pw_gid)) < 0 || (rc2 = setuid(new->pw_uid)) < 0) {
        fprintf(stderr, "Setting new uid (%d) and gid (%d) failed: %s\n", 
	        new->pw_uid, new->pw_gid, strerror(errno));
	fprintf(stderr, "Return codes: gid: %d; uid: %d\n", rc1, rc2);
	exit(1);
    }

    printf("After change:\n");
    printf("    Real UID: %5d; Effective UID: %5d\n", getuid(), geteuid());
    printf("    Real GID: %5d; Effective GID: %5d\n", getgid(), getegid());
}
